﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.Wpf;

namespace _9364
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void map_Loaded(object sender, RoutedEventArgs e)
        {
            map.MapUnit = GeographyUnit.DecimalDegree;

            LayerOverlay layerOverlay = new LayerOverlay();
            ShapeFileFeatureLayer shapeFileLayer = new ShapeFileFeatureLayer(@"..\..\AppData\states.shp");
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = WorldStreetsAreaStyles.Military();
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layerOverlay.Layers.Add(shapeFileLayer);
            map.Overlays.Add(layerOverlay);

            map.TrackOverlay = new RulerTrackInteractiveOverlay();
            map.TrackOverlay.TrackMode = TrackMode.Line;

            shapeFileLayer.Open();
            map.CurrentExtent = shapeFileLayer.GetBoundingBox();
            map.Refresh();
        }
    }

    public class RulerTrackInteractiveOverlay : TrackInteractiveOverlay
    {
        private const string lineFeatureKey = "lineFeature";
        private const string ellipseFeatureKey = "ellipseFeature";
        private LineShape rulerLineShape;
        private int mouseDown;

        public RulerTrackInteractiveOverlay()
            : base()
        {
            TrackShapeLayer.Open();
            TrackShapeLayer.Columns.Add(new FeatureSourceColumn("length"));
            //Sets the appearance of the ruler
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.FromArgb(150, GeoColor.StandardColors.DarkRed), 4, false);
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Red, 2);
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("length", new GeoFont("Arial", 10, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColor.SimpleColors.Black));
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.TextLineSegmentRatio = 100;
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.YOffsetInPixel = 10;
            TrackShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        }

        protected override InteractiveResult MouseDownCore(InteractionArguments interactionArguments)
        {
            //Sets the first and last point of the line where the user clicks.
            InteractiveResult interactiveResult = new InteractiveResult();
            interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
            interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;

            rulerLineShape = new LineShape(new Collection<Vertex>() { new Vertex(interactionArguments.WorldX, interactionArguments.WorldY), new Vertex(interactionArguments.WorldX, interactionArguments.WorldY) });

            TrackShapeLayer.InternalFeatures.Add(lineFeatureKey, new Feature(rulerLineShape));
            TrackShapeLayer.InternalFeatures.Add(ellipseFeatureKey, new Feature());

            mouseDown++;
            return interactiveResult;
        }

        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            InteractiveResult interactiveResult = new InteractiveResult();

            if (mouseDown > 0)
            {
                rulerLineShape.Vertices[rulerLineShape.Vertices.Count - 1] = new Vertex(interactionArguments.WorldX, interactionArguments.WorldY);

                TrackShapeLayer.InternalFeatures[lineFeatureKey] = new Feature(rulerLineShape);

                var lengthForLabel = rulerLineShape.GetLength(GeographyUnit.DecimalDegree, DistanceUnit.Feet);
                TrackShapeLayer.InternalFeatures[lineFeatureKey].ColumnValues["length"] = ((int)lengthForLabel).ToString() + " feet";

                var lengthForEllipse = rulerLineShape.GetLength(GeographyUnit.Meter, DistanceUnit.Meter);
                EllipseShape es = new EllipseShape(new PointShape(rulerLineShape.Vertices[0]), lengthForEllipse);
                TrackShapeLayer.InternalFeatures[ellipseFeatureKey] = new Feature(es);
                
                interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
                interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;
            }

            return interactiveResult;
        }

        protected override InteractiveResult MouseUpCore(InteractionArguments interactionArguments)
        {
            //Removes the line of the ruler at finishing dragging (at mouse up event).
            InteractiveResult interactiveResult = new InteractiveResult();

            interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw;
            interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays;

            if (mouseDown == 1)
            {
                mouseDown = 0;
                try
                {
                    TrackShapeLayer.InternalFeatures.Remove("test");
                }
                catch (Exception)
                {

                }
                TrackShapeLayer.InternalFeatures.Remove(lineFeatureKey);
                TrackShapeLayer.InternalFeatures.Remove(ellipseFeatureKey);
            }
            return interactiveResult;
        }
    }
}
